@@ -1,5 +1,5 @@
 /*  GNU ddrescue - Data recovery tool
-    Copyright (C) 2004-2016 Antonio Diaz Diaz.
+    Copyright (C) 2004-2017 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,44 +45,52 @@ public:
 
 struct Rb_options
   {
-  enum { default_skipbs = 65536, max_max_skipbs = 1 << 30 };
+  enum { min_skipbs = 65536 };
 
+  const long long max_max_skipbs;
   long long max_error_rate;
   long long min_outfile_size;
   long long max_read_rate;
-  long long min_read_rate;
-  long max_errors;
-  long pause;
-  long timeout;
-  int cpass_bitset;		// 1 | 2 | 4 for passes 1, 2, 3
+  long long min_read_rate;	// -2 = not set, -1 = reset
+  long long skipbs;		// initial size to skip on read error
+  long long max_skipbs;		// maximum size to skip on read error
+  unsigned long max_bad_areas;
+  unsigned long max_read_errors;
+  unsigned long max_slow_reads;
+  int cpass_bitset;		// 1 << ( pass - 1 ) for passes 1 to 5
+  int delay_slow;
   int max_retries;
   int o_direct_in;		// O_DIRECT or 0
+  Rational pause_on_error;
+  int pause_on_pass;
   int preview_lines;		// preview lines to show. 0 = disable
-  int skipbs;			// initial size to skip on read error
-  int max_skipbs;		// maximum size to skip on read error
+  int timeout;
   bool complete_only;
-  bool exit_on_error;
-  bool new_errors_only;
+  bool new_bad_areas_only;
   bool noscrape;
   bool notrim;
   bool reopen_on_error;
+  bool reset_slow;
   bool retrim;
   bool reverse;
+  bool simulated_poe;
   bool sparse;
   bool try_again;
   bool unidirectional;
   bool verify_on_error;
 
   Rb_options()
-    : max_error_rate( -1 ), min_outfile_size( -1 ), max_read_rate( 0 ),
-      min_read_rate( -1 ), max_errors( -1 ), pause( 0 ), timeout( -1 ),
-      cpass_bitset( 7 ), max_retries( 0 ), o_direct_in( 0 ),
-      preview_lines( 0 ), skipbs( default_skipbs ), max_skipbs( max_max_skipbs ),
-      complete_only( false ), exit_on_error( false ),
-      new_errors_only( false ), noscrape( false ), notrim( false ),
-      reopen_on_error( false ), retrim( false ), reverse( false ),
-      sparse( false ), try_again( false ), unidirectional( false ),
-      verify_on_error( false )
+    : max_max_skipbs( 1LL << 60 ), max_error_rate( -1 ), min_outfile_size( -1 ),
+      max_read_rate( 0 ), min_read_rate( -2 ), skipbs( -1 ),
+      max_skipbs( max_max_skipbs ), max_bad_areas( ULONG_MAX ),
+      max_read_errors( ULONG_MAX ), max_slow_reads( ULONG_MAX ),
+      cpass_bitset( 31 ), delay_slow( 30 ), max_retries( 0 ), o_direct_in( 0 ),
+      pause_on_error( 0 ), pause_on_pass( 0 ), preview_lines( 0 ),
+      timeout( -1 ), complete_only( false ), new_bad_areas_only( false ),
+      noscrape( false ), notrim( false ), reopen_on_error( false ),
+      reset_slow( false ), retrim( false ), reverse( false ),
+      simulated_poe( false ), sparse( false ), try_again( false ),
+      unidirectional( false ), verify_on_error( false )
       {}
 
   bool operator==( const Rb_options & o ) const
@@ -90,18 +98,24 @@ struct Rb_options
                min_outfile_size == o.min_outfile_size &&
                max_read_rate == o.max_read_rate &&
                min_read_rate == o.min_read_rate &&
-               max_errors == o.max_errors && pause == o.pause &&
-               timeout == o.timeout && cpass_bitset == o.cpass_bitset &&
+               skipbs == o.skipbs && max_skipbs == o.max_skipbs &&
+               max_bad_areas == o.max_bad_areas &&
+               max_read_errors == o.max_read_errors &&
+               max_slow_reads == o.max_slow_reads &&
+               cpass_bitset == o.cpass_bitset &&
+               delay_slow == o.delay_slow &&
                max_retries == o.max_retries &&
                o_direct_in == o.o_direct_in &&
-               preview_lines == o.preview_lines &&
-               skipbs == o.skipbs && max_skipbs == o.max_skipbs &&
+               pause_on_error == o.pause_on_error &&
+               pause_on_pass == o.pause_on_pass &&
+               preview_lines == o.preview_lines && timeout == o.timeout &&
                complete_only == o.complete_only &&
-               exit_on_error == o.exit_on_error &&
-               new_errors_only == o.new_errors_only &&
+               new_bad_areas_only == o.new_bad_areas_only &&
                noscrape == o.noscrape && notrim == o.notrim &&
                reopen_on_error == o.reopen_on_error &&
+               reset_slow == o.reset_slow &&
                retrim == o.retrim && reverse == o.reverse &&
+               simulated_poe == o.simulated_poe &&
                sparse == o.sparse && try_again == o.try_again &&
                unidirectional == o.unidirectional &&
                verify_on_error == o.verify_on_error ); }
@@ -112,17 +126,19 @@ struct Rb_options
 
 class Rescuebook : public Mapbook, public Rb_options
   {
-  long long error_rate;
+  long long error_rate, error_sum;
   long long sparse_size;		// end position of pending writes
   long long non_tried_size, non_trimmed_size, non_scraped_size;
-  long long bad_sector_size, finished_size;
+  long long bad_size, finished_size;
   const Domain * const test_domain;	// good/bad map for test mode
   const char * const iname_;
-  int e_code;				// error code for errors_or_timeout
-					// 1 rate, 2 errors, 4 timeout,
-					// 8 other (explained in final_msg)
-  long errors;				// error areas found so far
+  unsigned long bad_areas;		// bad areas found so far
+  unsigned long read_errors, slow_reads;
   int ides_, odes_;			// input and output file descriptors
+  int e_code;				// error code for errors_or_timeout
+					// 1 rate, 2 bad_areas, 4 timeout,
+					// 8 other (explained in final_msg),
+					// 16 read_errors, 32 slow_reads
   const bool synchronous_;
   long long voe_ipos;			// pos of last good sector read, or -1
   uint8_t * const voe_buf;		// copy of last good sector read
@@ -131,48 +147,47 @@ class Rescuebook : public Mapbook, public Rb_options
   long long iobuf_ipos;			// last pos read in iobuf, or -1
   long long last_ipos;
   long t0, t1, ts;			// start, current, last successful
+  Rational tp;				// cumulated pause_on_error
   int oldlen;
-  bool rates_updated;
+  bool rates_updated, current_slow, prev_slow;
   Sliding_average sliding_avg;		// variables for show_status
   bool first_post;			// first read in current pass
   bool first_read;			// first read overall
 
   void change_chunk_status( const Block & b, const Sblock::Status st );
+  void do_pause_on_error();
   bool extend_outfile_size();
   int copy_block( const Block & b, int & copied_size, int & error_size );
   void initialize_sizes();
   bool errors_or_timeout()
-    { if( max_errors >= 0 && errors > max_errors ) e_code |= 2;
-      return ( e_code != 0 ); }
-  void reduce_min_read_rate()
-    { if( min_read_rate > 0 ) min_read_rate /= 10; }
-  bool slow_read() const
-    { return ( t1 - t0 >= 30 &&		// no slow reads for first 30s
-               ( ( min_read_rate > 0 && c_rate < min_read_rate &&
-                   c_rate < a_rate / 2 ) ||
-                 ( min_read_rate == 0 && c_rate < a_rate / 10 ) ) ); }
+    { if( bad_areas > max_bad_areas ) e_code |= 2; return ( e_code != 0 ); }
+  const char * percent_rescued() const
+    { return format_percentage( finished_size, domain().in_size(), 3, 2 ); }
   int copy_and_update( const Block & b, int & copied_size,
                        int & error_size, const char * const msg,
-                       const Status curr_st, const bool forward,
+                       const Status curr_st, const int curr_pass,
+                       const bool forward,
                        const Sblock::Status st = Sblock::bad_sector );
   bool reopen_infile();
   int copy_non_tried();
-  int fcopy_non_tried( const char * const msg, const int pass );
-  int rcopy_non_tried( const char * const msg, const int pass );
+  int fcopy_non_tried( const char * const msg, const int pass,
+                       const bool resume );
+  int rcopy_non_tried( const char * const msg, const int pass,
+                       const bool resume );
   int trim_errors();
   int scrape_errors();
   int copy_errors();
-  int fcopy_errors( const char * const msg, const int retry );
-  int rcopy_errors( const char * const msg, const int retry );
-  void update_rates( const bool force = false );
+  int fcopy_errors( const char * const msg, const int pass, const bool resume );
+  int rcopy_errors( const char * const msg, const int pass, const bool resume );
+  bool update_rates( const bool force = false );
   void show_status( const long long ipos, const char * const msg = 0,
                     const bool force = false );
 public:
   Rescuebook( const long long offset, const long long isize,
               Domain & dom, const Domain * const test_dom,
-              const Rb_options & rb_opts, const char * const iname,
-              const char * const mapname, const int cluster,
-              const int hardbs, const bool synchronous );
+              const Mb_options & mb_opts, const Rb_options & rb_opts,
+              const char * const iname, const char * const mapname,
+              const int cluster, const int hardbs, const bool synchronous );
   ~Rescuebook() { delete[] voe_buf; }
 
   int do_rescue( const int ides, const int odes );
